//! # CRC's from scratch, nothing else
//!
//! This crate implements some CRC algorithms from scratch in a `no_std` environment.
//!
//! ## Acknowledgements
//!
//! Many thanks to the [crc-catalogue](https://reveng.sourceforge.io/crc-catalogue/) for providing
//! some test vectors.

#![cfg_attr(not(test), no_std)]

use num_traits::{AsPrimitive, Num, PrimInt, Unsigned};

mod crc_32;

pub const CHECK_DATA: [u8; 9] = *b"123456789";

pub use crc_32::*;

pub trait Crc {
    type Input: ?Sized;
    type Checksum: Eq + Default;

    fn new() -> Self;
    fn process(&mut self, data: &Self::Input);
    fn shift_reg(&mut self) -> &mut Self::Checksum;
}

pub fn bit_at<N: PrimInt + Unsigned + AsPrimitive<u8>>(mut n: N, idx: usize) -> u8
where
    u8: AsPrimitive<N>,
{
    n = n & (1.as_() << idx);
    (n >> idx).as_()
}

#[cfg(test)]
mod test {
    use crate::bit_at;

    #[test]
    fn test_bit_at() {
        let a: u8 = 0b11110000;
        assert_eq!(bit_at(a, 0), 0);
        assert_eq!(bit_at(a, 1), 0);
        assert_eq!(bit_at(a, 2), 0);
        assert_eq!(bit_at(a, 3), 0);
        assert_eq!(bit_at(a, 4), 1);
        assert_eq!(bit_at(a, 5), 1);
        assert_eq!(bit_at(a, 6), 1);
        assert_eq!(bit_at(a, 7), 1);

        let a: u16 = 0b1111000011110101;
        assert_eq!(bit_at(a, 0), 1);
        assert_eq!(bit_at(a, 1), 0);
        assert_eq!(bit_at(a, 2), 1);
        assert_eq!(bit_at(a, 3), 0);
        assert_eq!(bit_at(a, 4), 1);
        assert_eq!(bit_at(a, 5), 1);
        assert_eq!(bit_at(a, 6), 1);
        assert_eq!(bit_at(a, 7), 1);
        assert_eq!(bit_at(a, 8), 0);
        assert_eq!(bit_at(a, 9), 0);
        assert_eq!(bit_at(a, 10), 0);
        assert_eq!(bit_at(a, 11), 0);
        assert_eq!(bit_at(a, 12), 1);
        assert_eq!(bit_at(a, 13), 1);
        assert_eq!(bit_at(a, 14), 1);
        assert_eq!(bit_at(a, 15), 1);
    }
}
